@@ -1,7 +1,7 @@
 /** @file
  * @brief Unit tests of non-Xapian-specific internal code.
  */
-/* Copyright (C) 2006-2022 Olly Betts
+/* Copyright (C) 2006-2023 Olly Betts
  * Copyright (C) 2007 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
@@ -76,9 +76,11 @@ using namespace std;
 // Code we're unit testing:
 #include "../common/closefrom.cc"
 #include "../common/errno_to_string.cc"
+#include "../common/io_utils.cc"
 #include "../common/fileutils.cc"
 #include "../common/overflow.h"
 #include "../common/parseint.h"
+#include "../common/posixy_wrapper.cc"
 #include "../common/serialise-double.cc"
 #include "../common/str.cc"
 #include "../backends/uuids.cc"
@@ -218,9 +220,9 @@ check_double_serialisation(double u)
     const char * end = ptr[1] + encoded.size();
     double v = unserialise_double(&(ptr[1]), end);
     if (ptr[1] != end || u != v) {
-	cout << u << " -> " << v << ", difference = " << v - u << endl;
-	cout << "FLT_RADIX = " << FLT_RADIX << endl;
-	cout << "DBL_MAX_EXP = " << DBL_MAX_EXP << endl;
+	cout << u << " -> " << v << ", difference = " << v - u << '\n';
+	cout << "FLT_RADIX = " << FLT_RADIX << '\n';
+	cout << "DBL_MAX_EXP = " << DBL_MAX_EXP << '\n';
     }
     TEST_EQUAL(static_cast<const void*>(ptr[1]), static_cast<const void*>(end));
 }
@@ -267,7 +269,7 @@ static void test_serialiselength1()
 	const char *p_end = p + s.size();
 	size_t decoded_n;
 	decode_length(&p, p_end, decoded_n);
-	if (n != decoded_n || p != p_end) tout << "[" << s << "]" << endl;
+	if (n != decoded_n || p != p_end) tout << "[" << s << "]\n";
 	TEST_EQUAL(n, decoded_n);
 	TEST_EQUAL(p_end - p, 0);
 	if (n < 5000) {
@@ -416,9 +418,6 @@ static void test_log2()
 }
 
 static const double test_sortableserialise_numbers[] = {
-#ifdef INFINITY
-    -INFINITY,
-#endif
     -HUGE_VAL,
     -DBL_MAX,
     -exp2(1022),
@@ -460,9 +459,6 @@ static const double test_sortableserialise_numbers[] = {
     exp2(1022),
     DBL_MAX,
     HUGE_VAL,
-#ifdef INFINITY
-    INFINITY,
-#endif
 
     64 // Magic number which we stop at.
 };
@@ -512,7 +508,7 @@ template<typename S>
 inline static void tostring_helper() {
     const S max_val = numeric_limits<S>::max();
     const S min_val = numeric_limits<S>::min();
-    tout << "Testing with tostring_helper" << endl;
+    tout << "Testing with tostring_helper\n";
     std::ostringstream oss;
     oss << (long long)max_val;
     TEST_EQUAL(str(max_val), oss.str());
@@ -843,7 +839,7 @@ template<typename U>
 inline static void parseunsigned_helper() {
     U val;
     const U max_val = numeric_limits<U>::max();
-    tout << "Testing with parseunsigned_helper" << endl;
+    tout << "Testing with parseunsigned_helper\n";
     TEST(parse_unsigned("0", val));
     TEST_EQUAL(val, 0);
     TEST(parse_unsigned("99", val));
@@ -873,7 +869,7 @@ inline static void parsesigned_helper() {
     S val;
     const S max_val = numeric_limits<S>::max();
     const S min_val = numeric_limits<S>::min();
-    tout << "Testing with parsesigned_helper" << endl;
+    tout << "Testing with parsesigned_helper\n";
     TEST(parse_signed("0", val));
     TEST_EQUAL(val, 0);
     TEST(parse_signed("99", val));
@@ -907,6 +903,79 @@ static void test_parsesigned1()
     parsesigned_helper<long long>();
 }
 
+/// Test working with a block-based file using functions from io_utils.h.
+static void test_ioblock1()
+{
+    constexpr int BLOCK_SIZE = 1024;
+    const char* tmp_file = ".unittest_ioutils1";
+
+    int fd = io_open_block_wr(tmp_file, true);
+    TEST_REL(fd, >=, 0);
+
+    string buf(BLOCK_SIZE, 'x');
+    string out;
+
+    io_write_block(fd, buf.data(), BLOCK_SIZE, 1);
+    out.resize(BLOCK_SIZE);
+    io_read_block(fd, &out[0], BLOCK_SIZE, 1);
+    TEST(buf == out);
+
+    // Call io_sync() and check it claims to work.  Checking it actually has
+    // any effect is much harder to do.
+    TEST(io_sync(fd));
+
+    io_write_block(fd, buf.data(), BLOCK_SIZE, 12);
+    out.resize(BLOCK_SIZE);
+    io_read_block(fd, &out[0], BLOCK_SIZE, 12);
+    TEST(buf == out);
+
+    // Call io_full_sync() and check it claims to work.  Checking it actually
+    // has any effect is much harder to do.
+    TEST(io_full_sync(fd));
+
+    if (sizeof(off_t) <= 4) {
+	close(fd);
+	io_unlink(tmp_file);
+	SKIP_TEST("Skipping rest of testcase - off_t not 64-bit");
+    }
+
+#ifndef __WIN32__
+    struct stat statbuf;
+    TEST(fstat(fd, &statbuf) == 0);
+    TEST_REL(statbuf.st_blocks, >=, BLOCK_SIZE / 512 * 2);
+    if (statbuf.st_blocks >= BLOCK_SIZE / 512 * (12 + 1)) {
+	close(fd);
+	io_unlink(tmp_file);
+	SKIP_TEST("Skipping rest of testcase - FS doesn't support holes");
+    }
+
+    // Write a block before 4GB and check that we wrote the specified block by
+    // checking the filesize.  This should catch bugs which truncate the
+    // offset used.
+    io_write_block(fd, buf.data(), BLOCK_SIZE, (1ull << 32) - 1);
+    TEST(fstat(fd, &statbuf) == 0);
+    TEST_EQUAL(statbuf.st_size, (1ull << 32) * 1024);
+
+    close(fd);
+
+    fd = io_open_block_rd(tmp_file);
+
+    // We can't easily test that io_readahead_block() actually does anything if
+    // it returns true, but we can at least call it to check it doesn't crash.
+    (void)io_readahead_block(fd, BLOCK_SIZE, 12);
+
+    io_read_block(fd, &out[0], BLOCK_SIZE, (1ull << 32) - 1);
+    TEST(buf == out);
+
+    close(fd);
+    io_unlink(tmp_file);
+#else
+    close(fd);
+    io_unlink(tmp_file);
+    SKIP_TEST("Skipping rest of testcase - FS doesn't support holes");
+#endif
+}
+
 static const test_desc tests[] = {
     TESTCASE(simple_exceptions_work1),
     TESTCASE(class_exceptions_work1),
@@ -929,6 +998,7 @@ static const test_desc tests[] = {
     TESTCASE(muloverflows1),
     TESTCASE(parseunsigned1),
     TESTCASE(parsesigned1),
+    TESTCASE(ioblock1),
     END_OF_TESTCASES
 };
 
@@ -937,6 +1007,6 @@ try {
     test_driver::parse_command_line(argc, argv);
     return test_driver::run(tests);
 } catch (const char * e) {
-    cout << e << endl;
+    cout << e << '\n';
     return 1;
 }
